;**********************************************************************************************
;** Analog/Digitale-Wanduhr 2		 						     **
;**											     **
;** Wanduhr mit PIC16F84 und 72 LEDs, wobei 60 LEDs in Form eines regelm. 12-Ecks angeordnet **
;** sind. Diese zeigen die Minuten bzw. die Sekunden an. Die restlichen 12 LEDs sind auf die **
;** 12 Ecken aufgeteilt und zeigen die aktuelle Stunde an.				     **
;**											     **
;** Hardware: 										     **
;**	+ Mikrocontroller: PIC16F84, Takt (Quarz 4,096 MHz)				     **
;**	+ DCF-Modul (mit Anpasschaltung)						     **
;**	+ Ansteuerelektronik fr die LEDs						     **
;**											     **
;** Die Aufgabe der PIC-Software besteht bei der DCF-Dekodierung aus mehreren Teilaufgaben:  **
;**	+ In kurzen Zeitabstnden (hier alle 4 ms) das vom DCF-Empfangsmodul erzeugte Signal **
;**	  abtasten. Daraus entweder ein Low, High oder eine neue Minuten ermitteln. Kann aus **
;**	  den Abtastungen keines dieser drei Flle ermittelt werden, so handelt es sich um   **
;**	  einen Telegrammfehler.  							     **
;**	+ Die empfangenen Low- und High-Pegel zwischenspeichern				     **
;**	+ Bei jeder neuen Minute aus den gespeicherten Werten entsprechend die Zeit- und     **
;**	  Datumswerte zusammensetzen und in entsprechenden Registern sichern.		     **
;**	+ Die soeben gewonnen Zeit- und Datumsinformationen mit den Zeit- und Datums-	     **
;**	  informationen der vorhergehenden Minute vergleichen. Die Zeit- und Datumswerte der **
;**	  soeben empfangenen Minute sind nur dann gltig, wenn sie sich um maximal 1 von den **
;**	  Zeit- und Datumsinformationen der vorhergehenden Minute unterscheiden.	     **
;**	+ Parallel zur DCF-Uhr eine reine Softwareuhr erzeugen. Diese Softwareuhr ist        **
;**	  gltig, wenn keine korrekte Zeit vom DCF-Empfangsmodul empfangen wird. Mit anderen **
;**	  Worten. Die Softwareuhr wird stndig mit einer gltig dekodierten "DCF-Zeit"       **
;**	  synchronisiert. Dies gilt natrlich auch fr die Datumsinformationen.		     **
;**											     **
;** Das DCF-Dekodierverfahren:							 	     **
;**	Der DCF-I/O-Eingang, an dem das DCF-Empfangsmodul (mit einer Anpasschaltung) ange-   **
;**	schlossen ist, wird zyklisch (ca. alle 4 ms) vom Unterprogramm DCFROUTINE abgefragt. **
;**	Als Zeitbasis fr den 4-ms-Takt dient der Timer-0-Interrupt. Dieser kann je nach     **
;**	Anwendung entweder so eingestellt werden, dass er alle 4 ms einen Interrupt auslst, **
;**	oder er wird so eingestellt dass er z. B. alle 500us oder alle 1ms einen Interrupt   **
;**	erzeugt und ein Zhlregister zhlt die notwendige Anzahl an Interruptaufrufen bis    **
;**	zu einer Zeitbasis von 4 ms. Hier wird der Einfachheit halber die erstere Variante   **
;**	verwendet. Der Aufruf des Unterprogramms DCFROUTINE erfolgt hier aber nicht von der  **
;**	ISR (Interrupt-Service-Routine), sondern vom Hauptprogramm. Die ISR setzt nur alle   **
;**	4ms ein Flag zur Kennzeichnung, dass das Hauptprogramm das Unterprogramm DCFROUTINE  **
;**	aufrufen soll.									     **
;**											     **
;**	Die Aufgabe des Unterprogramms DCFROUTINE besteht darin, aus dem vom DCF-Empfangs-   **
;**	modul empfangenen Datenstrom die Informationen LOW, HIGH und den Minutenwechsel (also**
;**	das Ausbleiben der 59. Sekunde) zu dekodieren. fr diese Informationen befinden sich **
;**	im DCF-Statusregister (DCFSTATUS) entsprechende Flags.  			     **
;**	Weiters beinhaltet das Register DCFSTATUS noch den Zustand des DCF-Eingangs 4 ms vor **
;**	dem Aufruf des Unterprogramms DCFROUTINE. Mit Hilfe des aktuellen Zustands und des   **
;**	Zustands vor 4 ms kann nun erkannt werden, ob sich der Pegel am DCF-Eingang gendert **
;**	hat.										     **
;**	Das Hauptprogramm prft nun stndig die Flags DCFNEUESEK (wird zustzlich zu	     **
;**	DCFLOW bzw. DCFHIGH gesetzt, wenn eine gltige Sekunde empfangen wurde) und	     **
;**	DCFNEUEMIN, und ruft die zugehrigen Unterprogramme DCFUPSEKUNDE bzw. DCFUPMINUTE    **
;**	auf. Die Aufgabe des Unterprogramms DCFUPSEKUNDE ist es, die empfangenen Low- und    **
;**	High-Pegel in den Register DCFTELEGRAMM1 bis DCFTELEGRAMM8 zwischenzuspeichern.	     **
;**	Die Aufgabe des Unterprogramms DCFUPMINUTE ist dagegen etwas umfangreicher: Zunchst **
;**	die Zeit und das Datum aus den in den Registern DCFTELEGRAMM1 bis DCFTELEGRAMM8      **
;**	zwischengespeicherten Werten zusammensetzen. Anschlieend die soeben gewonnen Zeit-  **
;**	und Datumsinformationen mit den Zeit- und Datumsinformationen der vorhergehenden     **
;**	Minute vergleichen. Die Zeit- und Datumswerte der soeben empfangenen Minute sind nur **
;**	dann gltig, wenn sie sich um maximal 1 von den Zeit- und Datumsinformationen der    **
;**	vorhergehenden Minute unterscheiden.						     **
;**											     **
;**	Parallel zur Dekodierung des DCF-Eingangs erzeugt das Unterprogramm INNEREUHR eine   **
;**	reine Softwareuhr. Fr diesen Zweck ist eine genaue 1-Sekunden-Zeitbasis notwendig.  **
;**	Diese Zeitbasis wird von der schon erwhnten Timer-0-ISR (Interrupt Service Routine) **
;**	zustzlich zur 4ms-Zeitbasis erzeugt. Auch fr diese Zeitbasis wird in der ISR ein   **
;**	entsprechendes Flag gesetzt, und das Hauptprogramm ruft das Unterprogramm INNEREUHR  **
;**	auf, wenn dieses Flag gesetzt ist. Diese zustzliche Softwareuhr mag auf dem ersten  **
;**	Blick unntig erscheinen. Es hat sich aber gezeigt, dass ein DCF-Empfang oft auch    **
;**	ber eine lngere Zeit nicht mglich ist. In diesem Fall wrde die Uhr "stehen".     **
;**	Diese Zeit wird mit einer zustzlichen Softwareuhr so berbrckt, dass der	     **
;**	Betrachter der Uhr davon nichts bemerkt.					     **
;** 											     **
;** Entwickler: Buchgeher Stefan							     **
;** Entwicklungsbeginn der Software: 21. Oktober 2003					     **
;** Funktionsfhig seit: 23. Oktober 2003						     **
;** Letzte Bearbeitung: 26. Februar 2004						     **
;**********************************************************************************************

		List 	p=PIC16F84


;******************************** Register (in Registerseite 0) *******************************
TMR0		equ	1		;Timer0-Register
PC		equ	2		;Programmzhler
STAT		equ	3		;Statusregister
PORTA		equ	5		;PortA-Register
PORTB		equ	6		;PortB-Register
INTCON		equ	0B		;Interrupt-Register


;******************************** Register (in Registerseite 1) *******************************
OPTREG		equ	1		;OPTION-Register
TRISA		equ	5		;Richtungsregister PortA
TRISB		equ	6		;Richtungsregister PortB


;******************************** Eigene Register (in Registerseite 0) ************************
ISR_STAT_TEMP	equ	0C		;Zwischenspeicher des Statusregister der ISR
ISR_w_TEMP	equ	0D		;Zwischenspeicher des Arbeitsregister der ISR
FLAGSISRHP	equ	0E		;beinhaltet Botschaftsflags ISR -> HP
ZAEHLERISR1SEK	equ	0F		;Zhlregister fr 1-Sekunden-Zeitbasis
DCFSTATUS	equ	10		;DCF-Statusregister
DCFPULS		equ	11		;Impulszeit des DCF-Impulses
DCFPAUSE	equ	12		;Zeit zwischen 2 DCF-Impulsen
DCFTELEGRAMM1	equ	13		;in diese Register wird das DCF-Telegramm
DCFTELEGRAMM2	equ	14		;  im Sekunden-Takt eingelesen. Beginnt eine
DCFTELEGRAMM3	equ	15		;  neue Minute, so werden diese Register 
DCFTELEGRAMM4	equ	16		;  ausgewertet, und in die entsprechenden
DCFTELEGRAMM5	equ	17		;  DCF-Zeit-Datums-Register kopiert (z.B. in das
DCFTELEGRAMM6	equ	18		;  Register DCFMINBCD)
DCFTELEGRAMM7	equ	19		;Reserve
DCFTELEGRAMM8	equ	1A		;Reserve
DCFMINBCD	equ	1B		;Beinhaltet die aktuelle dekodierte Minute (BCD-Format)
DCFSTDBCD	equ	1C		;Beinhaltet die aktuelle dekodierte Stunde (BCD-Format)
DCFTAGBCD	equ	1D		;Beinhaltet den aktuell dekodierten Tag (BCD-Format)
DCFMONBCD	equ	1E		;Beinhaltet den aktuell dekodierten Monat (BCD-Format)
DCFJAHRBCD	equ	1F		;Beinhaltet das aktuell dekodierte Jahr (BCD-Format)
DCFWOCHENTAG	equ	20		;Beinhaltet den aktuell dekodierten Wochentag 
					;  (1=Montag, 2=Dienstag, usw., 7=Sonntag)
DCFZUSATZINFOS	equ	21		;Beinhaltet Zusatzinformationen (Sommer/Winterzeit,
					;  Reserveantenne usw.)
DCFMINALT	equ	22		;In den Registern DCFxxxALT befinden sich die Uhrzeit-
DCFSTDALT	equ	23		;  und Datumsinfos der vorangegangenen Minute. Diese
DCFTAGALT	equ	24		;  werden zur berprfung des neuen DCF-Telegramms
DCFMONALT	equ	25		;  bentigt
DCFJAHRALT	equ	26
DCFWTAGALT	equ	27
UHRSEKUNDE	equ	28		;Sekundenzhler der inneren quarzgesteuerten Uhr
UHRMINUTE	equ	29		;Minutenzhler der inneren quarzgesteuerten Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
UHRSTUNDE	equ	2A		;Stundenzhler der inneren quarzgesteuerten Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
DATUMWTAG	equ	2B		;Wochentagszhler der inneren quarzgest. Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
DATUMTAG	equ	2C		;Tageszhler der inneren quarzgesteuerten Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
DATUMMONAT	equ	2D		;Monatszhler der inneren quarzgesteuerten Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
DATUMJAHR	equ	2E		;Jahreszhler der inneren quarzgesteuerten Uhr, wird
					;  mit jedem gltigen DCF-dekod. Wert synchronisiert
FLAGS		equ	30

TEMP1		equ	4A		;allgemeines Hilfsregister 1
TEMP2		equ	4B		;allgemeines Hilfsregister 2


;******************************** Bits in Registern der Registerbank 0 ************************
;Register STAT
C		equ	0		;Carrybit im Statuswort-Register
Z		equ	2		;Zerobit im Statuswort-Register
RP0		equ	5		;Seitenauswahlbit im Statuswort-Register

;Register INTCON
T0IF		equ	2		;TMR0-Interruptflag im INTCON-Register
GIE		equ	7		;Interruptfreigabeflag


;******************************** Bits in den eigenen Registern der Registerbank 0 ************
;Register FLAGSISRHP
FLAG4MSEK	equ	0
FLAG1SEK	equ	1

;Register DCFSTATUS
DCFPORTNEU	equ	0		;Akt. Zustand DCF-Porteingang
DCFPORTALT	equ	1		;Vorhergehender Zustand am DCF-Porteingang
DCFNEUESEK	equ	2		;gesetzt, wenn neue Sekunde begonnen
DCFNEUEMIN	equ	3		;gesetzt, wenn neue Minute begonnen
DCFLOW		equ	4		;gesetzt, wenn Low-Signal erkannt
DCFHIGH		equ	5		;gesetzt, wenn High-Signal erkannt
DCFFEHLER	equ	6		;gesetzt, wenn ein Fehler erkannt
DCFSYNC		equ	7		;gesetzt, wenn Uhr mit DCF synchronisiert

;Register DCFZUSATZINFOS
DCFRESANT	equ	0		;Reserveantenne
DCFSOMWIN	equ	1		;Vorankndigung der Sommer/Winterzeit-Umstellung
DCFSOMMER	equ	2		;Sommerzeit
DCFWINTER	equ	3		;Winterzeit
DCFSCHALTSEK	equ	4		;Vorankndigung einer zustzliche Schaltsekunde


;******************************** Portbelegung ************************************************
;Port A	
DCFINPORT	equ	PORTA
DCFINTRIS	equ	TRISA
DCFIN		equ	0


;******************************** Konstanten **************************************************
KONSTISR1SEK	equ	.250
DCFLOWMIN	equ	.20
DCFLOWMAX	equ	.30
DCFHIGHMIN	equ	.40
DCFHIGHMAX	equ	.60


;******************************** Ziele der Registeroperationen *******************************
w		equ	0		;w ist Zielregister
f		equ	1		;f ist Zielregister


;******************************** Makros ******************************************************
;Umschalten zu Registerbank 0
bank0		MACRO
		bcf	STAT,RP0
		ENDM

;Umschalten zu Registerbank 1
bank1		MACRO
		bsf	STAT,RP0
		ENDM

;Bit invertieren (entspricht dem Befehl btg bei PIC17/18)
btg		MACRO	FILE,BIT
		local	SETBIT
		btfss	FILE,BIT
		goto	SETBIT
		bcf	FILE,BIT
		btfsc	FILE,BIT
SETBIT		bsf	FILE,BIT
		ENDM


;******************************** Konfigurations-Bits *****************************************
_lp_osc		equ	h'3FFC'
_xt_osc		equ	h'3FFD'
_hs_osc		equ	h'3FFE'
_rc_osc		equ	h'3FFF'

_wdt_off	equ	h'3FFB'
_wdt_on		equ	h'3FFF'

_pwrt_off	equ	h'3FFF'
_pwrt_on	equ	h'3FF7'

_cp_off		equ	h'3FFF'
_cp_on		equ	h'000F'

	__config	_hs_osc & _wdt_off & _pwrt_off & _cp_off

		ORG	0x000
		goto	Beginn
		ORG	0x004
		goto	ISR


;******************************** Tabellen ****************************************************

;**********************************************************************************************
;** Tabelle TABSTUNDEN:									     **
;** Ausgabetabelle der Stunde am Port A (Bits 1-4)					     **
;**********************************************************************************************
TABSTUNDEN	addwf	PC,f
		dt	.24		
		dt	.2,.4,.6,.8,.10,.12,.14,.16,.18,.20,.22,.24
		dt	.2,.4,.6,.8,.10,.12,.14,.16,.18,.20,.22 
		dt	.0,.0,.0,.0,.0,.0,.0,.0,.0


;**********************************************************************************************
;** Tabelle TABMINUTENSEKUNDEN:								     **
;** Ausgabetabelle der Minuten und Sekunden am Port B (Bits 0-5)			     **
;**********************************************************************************************
TABMINUTENSEKUNDEN
		addwf	PC,f
		dt	.59
		dt	.0,.1,.2,.3,.4,.5,.6,.7,.8,.9,.10,.11,.12
		dt	.13,.14,.15,.16,.17,.18,.19,.20,.21,.22,.23,.24
		dt	.25,.26,.27,.28,.29,.30,.31,.32,.33,.34,.35,.36
		dt	.37,.38,.39,.40,.41,.42,.43,.44,.45,.46,.47,.48
		dt	.49,.50,.51,.52,.53,.54,.55,.56,.57,.58
		dt	.60,.60,.60,.60,.60


;******************************** ISR - Timer0 ************************************************

;**********************************************************************************************
;** Interrupt Service Routine:				 				     **
;** 											     **
;** Aufruf: 										     **
;**	alle 4 ms									     **
;**											     **
;** Aufgaben:										     **
;**	+ w-Register (=Arbeitsregister) und Status-Register zwischenspeichern (PUSH)	     **
;**	+ Zeitbasis fr 4ms und 1 Sekunde erzeugen 					     **
;**	+ Das Timer-Interrupt-Flag T0IF wieder lschen					     **
;**	+ w-Register (=Arbeitsregister) und Statusregister wiederherstellen (POP).	     **
;**********************************************************************************************
ISR
PUSH		movwf	ISR_w_TEMP		;w-Register retten
		swapf	STAT,w			;Statusregister
		movwf	ISR_STAT_TEMP		;  retten

		;Beginn der eigentlichen ISR-Routine
		bsf	FLAGSISRHP,FLAG4MSEK	;Botschaftsflag setzen

		decfsz	ZAEHLERISR1SEK,f	;Zhlregister fr 1-Sekunden-Zeitbasis um 1
						;  vermindern

		goto	ISRWEITER1
		bsf	FLAGSISRHP,FLAG1SEK	;Botschaftsflag setzen
		movlw	KONSTISR1SEK		;Zhlregister fr den Sekundentakt mit
		movwf	ZAEHLERISR1SEK		;  der Konstanten KONSTISR1SEK laden

ISRWEITER1
		;Ende der eigentlichen ISR-Routine
ISRFERTIG	bcf	INTCON,T0IF		;T0-Interruptflag lschen

POP		swapf	ISR_STAT_TEMP,w	;Status-Register
		movwf	STAT			;  und
		swapf	ISR_w_TEMP,f		;  w-Register
		swapf	ISR_w_TEMP,w		;  wieder herstellen

		retfie


;******************************** Unterprogramme **********************************************

;**********************************************************************************************
;** Initialisierung des Prozessor:							     **
;**	+ TMR0-ISR soll alle 4 ms aufgerufen werden, dazu Vorteiler mit 16 laden (Bei einem  **
;**	  4,096-MHz-Quarz)								     **
;**	+ Ports: Port A: Ausgnge (Stundenanzeige)					     **
;**		 Port B: RB0 bis RA5: Ausgnge (Minutenanzeige)				     **
;**			 RB6, RB7:    Reserve						     **
;**		 (Der DCF-Eingang wird separat mit dem Befehl  bsf	DCFINTRIS,DCFIN als  **
;**		 Eingang definiert)							     **
;**	+ Zhlregister fr den Sekundentakt vorbelegen					     **
;**	+ DCF-Statusregister (DCFSTATUS) initialisieren (Fehlerflag setzen, alle anderen     **
;**	  Flags lschen)								     **
;**	+ Pegel von DCFIN einlesen und im Flag DCFPORTNEU des Register DCFSTATUS sichern     **
;**	+ Anzeige lschen								     **
;**	+ diverse Register vorbelegen (initialisieren)					     **
;**********************************************************************************************
INIT		clrf	TMR0			;Timer0 auf 0 voreinstellen

		bank1				;Registerseite 1
		movlw	b'00000011'		;interner Takt, Vorteiler = 16 an TMR0
		movwf	OPTREG
		clrf	TRISA			;Port A: Ausgnge (Stundenanzeige)

		movlw	b'00000000'		;Port B: Ausgang
		movwf	TRISB	

		bsf	DCFINTRIS,DCFIN		;DCFIN als Eingang definieren
		bank0				;Registerseite 0

		movlw	KONSTISR1SEK		;Zhlregister fr den Sekundentakt mit
		movwf	ZAEHLERISR1SEK		;  der Konstanten KONSTISR1SEK laden

		clrf	DCFSTATUS		;DCF-Statusregister (DCFSTATUS) initialisieren
		bsf	DCFSTATUS,DCFFEHLER	;  Fehlerflag setzen, alle anderen Flags
						;  lschen
	
		btfsc	DCFINPORT,DCFIN		;Pegel von DCFIN einlesen und im Flag
		bsf	DCFSTATUS,DCFPORTNEU	;  DCFPORTNEU des Register DCFSTATUS sichern

		movlw	b'11110'		;Anzeige lschen, dazu Werte am Port A und
		movwf	PORTA			;  am Port B ausgeben, die keiner Uhrzeit
		movlw	b'00111111'		;  entsprechen
		movwf	PORTB

		clrf	UHRSEKUNDE		;Sekundenzhler initialisieren (mit 0)
		clrf	UHRMINUTE		;Minutenzhler initialisieren (mit 0)
		clrf	UHRSTUNDE		;Stundenzhler initialisieren (mit 0)
		clrf	DATUMWTAG		;Wochentagzhler initialisieren (mit 0)
		clrf	DATUMTAG		;Tageszhler initialisieren (mit 0)
		clrf	DATUMMONAT		;Monatszhler initialisieren (mit 0)
		clrf	DATUMJAHR		;Jahreszhler initialisieren (mit 0)

		movlw	.99
		movwf	DCFMINALT		;Vergleichsregister initialisieren (mit 99)
		movwf	DCFSTDALT		;Vergleichsregister initialisieren (mit 99)
		movwf	DCFTAGALT		;Vergleichsregister initialisieren (mit 99)
		movwf	DCFMONALT		;Vergleichsregister initialisieren (mit 99)
		movwf	DCFJAHRALT		;Vergleichsregister initialisieren (mit 99)
		movwf	DCFWTAGALT		;Vergleichsregister initialisieren (mit 99)
		return


;******************************** DCF Routinen ************************************************

;**********************************************************************************************
;** DCFROUTINE:							 			     **
;** 											     **
;** Das Unterprogramm DCFROUTINE ist fr die DCF-Dekodierung die wichtigste Komponente.	     **
;**											     **
;** Diese Routine wird ca. alle 4 ms aufgerufen						     **
;**											     **
;** Aufgabe und Vorgehensweise: 							     **
;** 	Die Aufgabe des Unterprogramms DCFROUTINE besteht darin aus den Abtastungen heraus-  **
;**	zufinden, ob ein Low, ein High oder ein Minutenwechsel gesendet wurde. Dazu wird bei **
;**	jedem Aufruf des Unterprogramms DCFROUTINE entweder das Zhlregister DCFPULS oder    **
;**	das Zhlregister DCFPAUSE um 1 erhht (inkrementiert), wenn sich der Pegel vom DCF-  **
;**	Eingang zum vorhergehenden Aufruf dieses Unterprogramms nicht verndert hat. Ist     **
;**	der DCF-Eingang Low (also logisch 0), so wird das Zhlregister DCFPULS um 1 erhht,  **
;**	ist der DCF-Eingang High (also logisch 1), so wird das Zhlregister DCFPAUSE um 1    **
;**	erhht. Beide Zhlregister knnen maximal den Wert 255 aufnehmen, was einer Zeit     **
;**	von 1020 ms (oder 1,02 Sekunden) entspricht (=255 * 4ms).			     **
;**	Unterscheidet sich der aktuelle Pegel des DCF-Eingangs mit dem Pegel vom vorher-     **
;**	gehenden Aufruf, so spricht man von einer Flanke, wobei hier zwischen einer fallenden**
;**	und einer steigenden Flanke unterschieden werden muss. Bei einer fallenden Flanke    **
;**	(der aktuelle Pegel des DCF-Eingangs ist logisch 0), erfolgt Zunchst eine Aus-	     **
;**	wertung des Zhlregisters DCFPULS. Beinhaltet dieses Zhlregister einen Wert	     **
;**	zwischen den Konstanten DCFLOWMIN und DCFLOWMAX, so wurde ein LOW des DCF-Telegramms **
;**	ermittelt. Im DCF-Statusregister (DCFSTATUS) werden daher die Flags DCFLOW und	     **
;**	DCFNEUESEK gesetzt. Das Flag DCFNEUESEK dient als Zeichen einer neu empfangenen und  **
;**	gltigen Sekunde. Anschlieend wird das Zhlregister DCFPULS gelscht. Beinhaltet    **
;**	das Zhlregister DCFPULS einen Wert zwischen den Konstanten DCFHIGHMIN und	     **
;**	DCFHIGHMAX, so wurde ein HIGH des DCF-Telegramms ermittelt. Im DCF-Statusregister    **
;**	(DCFSTATUS) werden daher die Flags DCFHIGH und DCFNEUESEK gesetzt. Das Flag	     **
;**	DCFNEUESEK dient auch hier als Zeichen einer neu empfangenen und gltigen Sekunde,   **
;**	und auch das Zhlregister DCFPULS wird Anschlieend gelscht. Beinhaltet das	     **
;**	Zhlregister DCFPULS einen Wert der weder zwischen den Konstanten DCFLOWMIN und	     **
;**	DCFLOWMAX noch zwischen den Konstanten DCFHIGHMIN und DCFHIGHMAX liegt, so handelt es**
;**	sich um einen bertragungsfehler. In diesem Fall wird das Fehlerflag (DCFFEHLER) im  **
;**	DCF-Statusregister (DCFSTATUS) gesetzt und auch das Zhlregister DCFPULS wird an-    **
;**	schlieend gelscht. Das Flag DCFNEUESEK wird hier aber nicht gesetzt, da es sich in **
;**	diesem Fall um keine gltige Sekunde handelt. Das Fehlerflag wird erst bei der	     **
;**	Erkennung einer neuen Minute wieder gelscht. Achtung: Solange das Fehlerflag ge-    **
;**	setzt ist erfolgt keine Auswertung des Zhlregisters DCFPULS. Dieses Zhlregister    **
;**	wird aber dennoch bei jeder fallenden Flanke zurckgesetzt.			     **
;**	Bei einer steigenden Flanke (der aktuelle Pegel des DCF-Eingangs ist logisch 1) wird **
;**	das Zhlregister DCFPAUSE gelscht.						     **
;**	In der 59. Sekunde erfolgt keine Absenkung des Trgers. Da es also in der 59.	     **
;**	Sekunde keine fallende Flanke gibt, gibt es auch keine steigende Flanke. Das Zhl-   **
;**	register DCFPAUSE luft ber, da es nur einen Zhlbereich von 0 bis 255 besitzt.     **
;**	Dieser berlauf wird hier ausgentzt. Zur Bestimmung einer neuen Minute bzw. zur     **
;**	Bestimmung des Telegrammbeginns. Ist das Fehlerflag gesetzt, so wird es nun ge-	     **
;**	lscht. War es nicht gesetzt, so wird das Flag DCFNEUEMIN im DCF-Statusregister	     **
;**	(DCFSTATUS) gesetzt.								     **
;**											     **
;** Anmerkung:										     **
;** 	Die temporren Register TEMP1 und TEMP2 werden hier nur als Hilfsregister bentigt.  **
;** 	Sie knnen daher auch in anderen Unterprogrammen verwendet werden.		     **
;**********************************************************************************************
DCFROUTINE	bcf	DCFSTATUS,DCFPORTALT	;DCFSTATUS,DCFPORTNEU -> DCFSTATUS,DCFPORTALT
		btfsc	DCFSTATUS,DCFPORTNEU
		bsf	DCFSTATUS,DCFPORTALT

		movf	DCFINPORT,w
		movwf	TEMP1
		bcf	DCFSTATUS,DCFPORTNEU	;DCFIN -> DCFSTATUS,DCFPORTNEU, TEMP1,DCFPORTALT
		btfsc	TEMP1,DCFIN
		bsf	DCFSTATUS,DCFPORTNEU
		clrf	TEMP2
		btfsc	TEMP1,DCFIN
		bsf	TEMP2,DCFPORTALT

		movf	DCFSTATUS,w
		xorwf	TEMP2,f

		btfss	TEMP2,DCFPORTALT	;Hat sich am Pin DCFIN der Pegel gendert?
		goto	DCFINCPULSPAUSE		;nein: Puls- bzw. Pausezhler um 1 erhhen
		btfss	DCFSTATUS,DCFPORTNEU	;ja: -> steigende oder fallende Flanke?
		goto	DCFHLFLANKE		;fallende Flanke
DCFLHFLANKE	clrf	DCFPAUSE		;steigende Flanke: Zhlregister DCFPAUSE
		goto	DCFROUTINEFERTIG 	;  lschen und UP verlassen

		;Fallende Flanke
DCFHLFLANKE	btfss	DCFSTATUS,DCFFEHLER	;Fehlerflag gesetzt:
		goto	DCFPLAUSIBEL		;nein: Ermittlung, ob die ermittelte Pulszeit
						;  einem HIGH oder einem LOW entspricht
						;  (Plausibilittsprfung)
		goto	DCFPULSLOESCHEN		;ja: Puls-Zhler lschen
DCFPLAUSIBEL
		;Prfen, ob ein High empfangen wurde
DCFCHECKHIGH	movlw	DCFHIGHMIN		;unteren High-Grenzwert laden
		subwf	DCFPULS,w
		btfss	STAT,C			;Grenzwert < DCFPULS ?
		goto	DCFCHECKLOW		;nein: kein HIGH-Pegel, Prfen, ob LOW
		movlw	DCFHIGHMAX		;ja: DCFPULS mit oberen HIGH-Grenzwert
						;  vergleichen
		subwf	DCFPULS,w
		btfsc	STAT,C			;DCFPULS < Grenzwert ?
		goto	DCFCHECKLOW		;nein: es handelt sich um kein HIGH-Signal
						;  Prfen, ob ein LOW empfangen wurde
		bcf	DCFSTATUS,DCFLOW	;ja: Es wurde ein High-Signal empfangen,
						;  das Flag DCFLOW lschen, 
		bsf	DCFSTATUS,DCFHIGH	;  das Flag DCFHIGH setzen,
		bsf	DCFSTATUS,DCFNEUESEK	;  das Flag DCFNEUESEK setzen
		goto	DCFPULSLOESCHEN		;  und Puls-Zhler lschen

		;Prfen, ob ein Low empfangen wurde
DCFCHECKLOW	movlw	DCFLOWMIN		;unteren Low-Grenzwert laden
		subwf	DCFPULS,w
		btfss	STAT,C			;Grenzwert < DCFPULS ?
		goto	DCFSETFEHLER		;nein: kein LOW-Pegel, Empfangsfehler
		movlw	DCFLOWMAX		;ja: DCFPULS mit oberen LOW-Grenzwert
						;  vergleichen
		subwf	DCFPULS,w
		btfsc	STAT,C			;DCFPULS < Grenzwert ?
		goto	DCFSETFEHLER		;nein: es handelt sich um kein LOW-Signal
						;  -> Empfangsfehler
		bcf	DCFSTATUS,DCFHIGH	;ja: Es wurde ein LOW-Signal empfangen,
						;  das Flag DCFLOW setzen,
		bsf	DCFSTATUS,DCFLOW	;  das Flag DCFHIGH lschen,
		bsf	DCFSTATUS,DCFNEUESEK	;  das Flag DCFNEUESEK setzen
		goto	DCFPULSLOESCHEN		;  und den Puls-Zhler lschen

		;Empfangsfehler
DCFSETFEHLER	bsf	DCFSTATUS,DCFFEHLER	;Bei einem Empfangsfehler
		bcf	DCFSTATUS,DCFHIGH	;  Fehlerflag (DCFFEHLER) setzen,
		bcf	DCFSTATUS,DCFLOW	;  die Flags DCFLOW und DCFHIGH lschen,
		goto	DCFPULSLOESCHEN		;  und den Puls-Zhler lschen

		;Puls- oder Pausenzhler erhhen
DCFINCPULSPAUSE btfsc	DCFSTATUS,DCFPORTNEU
		goto	DCFINCPAUSE

DCFINCPULS	incf	DCFPULS,f		;Puls-Zhler um 1 erhhen
		goto	DCFROUTINEFERTIG

DCFINCPAUSE	incfsz	DCFPAUSE,f		;Pausezhler erhhen und auf berlauf
						;  Prfen
		goto	DCFROUTINEFERTIG	;kein berlauf: Unterprogramm verlassen

		btfsc	DCFSTATUS,DCFFEHLER	;berlauf: NEUE MINUTE: wenn Fehlerflag
		goto	DCFROUTINEW1

		bsf	DCFSTATUS,DCFNEUEMIN	;nicht gesetzt, Flag DCFNEUEMIN setzen
		goto	DCFROUTINEFERTIG	;  und Unterprogramm verlassen

DCFROUTINEW1	bcf	DCFSTATUS,DCFFEHLER	;Fehlerflag gesetzt: Fehlerflag lschen
		goto 	DCFROUTINEFERTIG	;  und Unterprogramm verlassen (Flag DCFNEUEMIN
						;  wird in diesem Fall nicht gesetzt)

		;Pulszhler lschen
DCFPULSLOESCHEN	clrf	DCFPULS

DCFROUTINEFERTIG
		return


;**********************************************************************************************
;** Neue Sekunde:									     **
;** Aufgaben: 										     **
;**	+ je nachdem ob das Bit DCFLOW oder das Bit DCFHIGH gesetzt ist dieses Bit dem 	     **
;**	  Telegramm (Register DCFTELEGRAMM1 bis DCFTELEGRAMM8) hinzufgen		     **
;**	+ Anforderungsflag (Flag DCFNEUESEK) zurcksetzen				     **
;**											     **
;** Vorgehensweise:									     **
;**	Ist das Flag DCFLOW (im Register DCFSTATUS) gesetzt das Carryflag (im SFR STAT)	     **
;**	lschen, ist aber das Flag DCFHIGH (ebenfalls im Register DCFSTATUS) gesetzt das     **
;**	Carryflag setzten. Dieses Carryflag nun dem Telegramm hinzufgen. Dieser Vorgang     **
;**	erfolgt mit einem so genannten Schiebebefehl. Dabei werden alle Bits des Registers   **
;**	DCFTELEGRAMM1 auf die nchst hhere Position verschoben. (Bit 6 wandert ins Bit 7,   **
;**	Bit 5 wandert ins Bit 6 usw. Bit 0 wandert ins Bit 1). Der Inhalt vom Carryflag	     **
;**	wandert ins Bit 0. Der Inhalt von Bit 7 wird ins Carryflag geschoben. Bei den	     **
;**	Register DCFTELEGRAMM2 bis DCFTELEGRAMM6 erfolgt derselbe Vorgang. (Der Inhalt von   **
;**	Bit 7 des Registers DCFTELEGRAMM1 wird ins Carry geschoben, und das Carry aber weiter**
;**	in das Bit 0 des Registers DCFTELEGRAMM2)					     **
;**											     **
;** Anmerkung: 										     **
;**	Die Flags DCFLOW und DCFHIGH knnen nie gleichzeitig gesetzt sein. Es ist nur	     **
;**	mglich, dass entweder nur DCFLOW oder nur DCFHIGH gesetzt ist, aber nie beide	     **
;**	gleichzeitig.									     **
;**********************************************************************************************
DCFUPSEKUNDE	btfsc	DCFSTATUS,DCFLOW  	;Ist das Flag DCFLOW im Register DCFSTATUS
						;  gesetzt?
		goto	DCFSCHIEBELOW
		btfsc	DCFSTATUS,DCFHIGH	;nein: Wenn das FLAG DCFHIGH im Register 
DCFSCHIEBEHIGH	bsf	STAT,C			;  DCFSTATUS gesetzt ist, dem Telegramm ein
		goto	DCFSCHIEBE		;  Low mit Hilfe des Carry hinzufgen
DCFSCHIEBELOW	bcf	STAT,C			;ja: Dem Telegramm ein High mit Hilfe des Carry
						;  hinzufgen
DCFSCHIEBE	rlf	DCFTELEGRAMM1,f
		rlf	DCFTELEGRAMM2,f
		rlf	DCFTELEGRAMM3,f
		rlf	DCFTELEGRAMM4,f
		rlf	DCFTELEGRAMM5,f
		rlf	DCFTELEGRAMM6,f
;		rlf	DCFTELEGRAMM7,f
;		rlf	DCFTELEGRAMM8,f

		bcf	DCFSTATUS,DCFNEUESEK 	;Anforderungsflag lschen
		return


;**********************************************************************************************
;** Neue Minute:						 			     **
;** Aufgaben: 										     **
;**	+ Fehlende 59. Sekunde "nachholen" (Unterprogramm DCFUPSEKUNDE aufrufen)	     **
;**	+ Datum, Uhrzeit und die Zusatzinformationen aus den Registern DCFTELEGRAMM1 bis     **
;**	  DCFTELEGRAMM8 zusammensetzen 							     **
;**	+ Das soeben ermittelte Telegramm mit dem Telegramm der vorhergehenden Minute ver-   **
;**	  gleichen. Die soeben ermittelte Minute muss dabei um 1 grer als die vorher-	     **
;**	  gehende Minute sein, und die soeben ermittelten Werte fr die Stunde, Tag, Monat,  **
;**	  Jahr und Wochentag muss mit den Werten des vorhergehenden Telegramms berein-	     **
;**	  stimmen. ACHTUNG: Bei dieser einfachen berprfung wird der bergang von z. B.     **
;**	  13:59 auf 14:00 als falsch gewertet, da sich hier die Stunde ndert, und auch die  **
;**	  Minute um mehr als 1. Wrden hier alle mglichen bergnge bercksichtigt,	     **
;**	  dann wrde dieses Unterprogramm noch umfangreicher als es ohnehin schon ist. Durch **
;**	  diese Vereinfachung dauert die Synchronisierung im schlimmsten Fall nur eine Minute**
;**	  lnger.  									     **
;**	+ Das alte Datum bzw. die alte Uhrzeit (von der vorhergehenden Minute) durch das neue**
;**	  Datum bzw. Uhrzeit ersetzen. Dies ist fr die berprfung des nchsten Telegramms  **
;**	  notwendig.									     **
;**	+ Wenn das neu empfangene Datum bzw. die neu empfangene Uhrzeit gltig ist, die BCD- **
;**	  kodierten Datums- bzw. Uhrzeitkomponenten in eine binre Form umwandeln und damit  **
;**	  die mitlaufende Uhr synchronisieren. Das Flag DCFSYNC im Register DCFSTATUS setzen.**
;**	  Dieses gesetzte Flag kennzeichnet, dass die mitlaufende Uhr mit der DCF-Uhr	     **
;**	  synchronisiert ist								     **
;**	+ Anforderungsflag (Flag DCFNEUEMIN im Register DCFSTATUS) zurcksetzen		     **
;**											     **
;** Anmerkung:										     **
;**	Das temporre Register TEMP1 wird hier nur als Hilfsregister bentigt, und kann	     **
;**	daher auch in anderen Unterprogrammen verwendet werden.				     **
;**********************************************************************************************
DCFUPMINUTE	call 	DCFUPSEKUNDE		;Fehlende Sekunde nachholen

		clrf	DCFJAHRBCD		;Jahr zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM1,1
		bsf	DCFJAHRBCD,7
		btfsc	DCFTELEGRAMM1,2
		bsf	DCFJAHRBCD,6
		btfsc	DCFTELEGRAMM1,3
		bsf	DCFJAHRBCD,5
		btfsc	DCFTELEGRAMM1,4
		bsf	DCFJAHRBCD,4
		btfsc	DCFTELEGRAMM1,5
		bsf	DCFJAHRBCD,3
		btfsc	DCFTELEGRAMM1,6
		bsf	DCFJAHRBCD,2
		btfsc	DCFTELEGRAMM1,7
		bsf	DCFJAHRBCD,1
		btfsc	DCFTELEGRAMM2,0
		bsf	DCFJAHRBCD,0

		clrf	DCFMONBCD		;Monat zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM2,1
		bsf	DCFMONBCD,4
		btfsc	DCFTELEGRAMM2,2
		bsf	DCFMONBCD,3
		btfsc	DCFTELEGRAMM2,3
		bsf	DCFMONBCD,2
		btfsc	DCFTELEGRAMM2,4
		bsf	DCFMONBCD,1
		btfsc	DCFTELEGRAMM2,5
		bsf	DCFMONBCD,0

		clrf	DCFWOCHENTAG		;Wochentag zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM2,6
		bsf	DCFWOCHENTAG,2
		btfsc	DCFTELEGRAMM2,7
		bsf	DCFWOCHENTAG,1
		btfsc	DCFTELEGRAMM3,0
		bsf	DCFWOCHENTAG,0

		clrf	DCFTAGBCD		;Tag zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM3,1
		bsf	DCFTAGBCD,5
		btfsc	DCFTELEGRAMM3,2
		bsf	DCFTAGBCD,4
		btfsc	DCFTELEGRAMM3,3
		bsf	DCFTAGBCD,3
		btfsc	DCFTELEGRAMM3,4
		bsf	DCFTAGBCD,2
		btfsc	DCFTELEGRAMM3,5
		bsf	DCFTAGBCD,1
		btfsc	DCFTELEGRAMM3,6
		bsf	DCFTAGBCD,0

		clrf	DCFSTDBCD		;Stunde zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM4,0
		bsf	DCFSTDBCD,5
		btfsc	DCFTELEGRAMM4,1
		bsf	DCFSTDBCD,4
		btfsc	DCFTELEGRAMM4,2
		bsf	DCFSTDBCD,3
		btfsc	DCFTELEGRAMM4,3
		bsf	DCFSTDBCD,2
		btfsc	DCFTELEGRAMM4,4
		bsf	DCFSTDBCD,1
		btfsc	DCFTELEGRAMM4,5
		bsf	DCFSTDBCD,0

		clrf	DCFMINBCD		;Minute zusammensetzen (BCD-Format)
		btfsc	DCFTELEGRAMM4,7
		bsf	DCFMINBCD,6
		btfsc	DCFTELEGRAMM5,0
		bsf	DCFMINBCD,5
		btfsc	DCFTELEGRAMM5,1
		bsf	DCFMINBCD,4
		btfsc	DCFTELEGRAMM5,2
		bsf	DCFMINBCD,3
		btfsc	DCFTELEGRAMM5,3
		bsf	DCFMINBCD,2
		btfsc	DCFTELEGRAMM5,4
		bsf	DCFMINBCD,1
		btfsc	DCFTELEGRAMM5,5
		bsf	DCFMINBCD,0

;		clrf	DCFZUSATZINFOS		;zustzliche Informationen
;		btfsc	DCFTELEGRAMM5,7		;  zustzliche Schaltsekunde
;		bsf	DCFZUSATZINFOS,DCFSCHALTSEK
;		btfsc	DCFTELEGRAMM6,0		;  Sommerzeit
;		bsf	DCFZUSATZINFOS,DCFSOMMER
;		btfsc	DCFTELEGRAMM6,1		;  Winterzeit
;		bsf	DCFZUSATZINFOS,DCFWINTER
;		btfsc	DCFTELEGRAMM6,2		;  Vorankndigung: Wechsel
;		bsf	DCFZUSATZINFOS,DCFSOMWIN ;   Sommerzeit <-> Winterzeit
;		btfsc	DCFTELEGRAMM6,3		;  Reserveantenne
;		bsf	DCFZUSATZINFOS,DCFRESANT

		;Neues Telegramm mit dem alten Vergleichen
		bcf	DCFSTATUS,DCFFEHLER	;zuerst Fehlerflag lschen

		movf	DCFMINALT,w		;Minuten Prfen
		subwf	DCFMINBCD,w
		movwf	TEMP1			;TEMP1 = DCFMINBCD - DCFMINALT
		decfsz	TEMP1,w			;TEMP1 - 1 = 0 ?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		movf	DCFSTDALT,w		;Stunden Prfen
		subwf	DCFSTDBCD,w
		btfss	STAT,Z			;DCFSTDBCD - DCFSTDALT = 0?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		movf	DCFTAGALT,w		;Tag Prfen
		subwf	DCFTAGBCD,w
		btfss	STAT,Z			;DCFTAGBCD - DCFTAGBCD = 0?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		movf	DCFMONALT,w		;Monat Prfen
		subwf	DCFMONBCD,w
		btfss	STAT,Z			;DCFMONBCD - DCFMONALT = 0?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		movf	DCFJAHRALT,w		;Jahr Prfen
		subwf	DCFJAHRBCD,w
		btfss	STAT,Z			;DCFJAHRBCD - DCFJAHRALT = 0?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		movf	DCFWTAGALT,w		;Wochentag Prfen
		subwf	DCFWOCHENTAG,w
		btfss	STAT,Z			;DCFWOCHENTAG - DCFWTAGALT = 0?
		bsf	DCFSTATUS,DCFFEHLER	;nein: Fehlerflag setzen

		;Altes Telegramm durch das neue Telegramm ersetzen
		movf	DCFMINBCD,w
		movwf	DCFMINALT

		movf	DCFSTDBCD,w
		movwf	DCFSTDALT

		movf	DCFTAGBCD,w
		movwf	DCFTAGALT

		movf	DCFMONBCD,w
		movwf	DCFMONALT

		movf	DCFJAHRBCD,w
		movwf	DCFJAHRALT

		movf	DCFWOCHENTAG,w
		movwf	DCFWTAGALT

		btfsc	DCFSTATUS,DCFFEHLER	;DCF-Telegramm korrekt?
		goto	DCFUPMINUTEENDE		;nein: UP beenden
		movf	DCFSTDBCD,w 		;ja: DCFSTDBCD
		call	BCDBIN2			;von BCD nach binr umwandeln
		movwf	UHRSTUNDE		;und die Stunde der mitlaufenden Uhr
						;berschreiben

		movf	DCFMINBCD,w		;DCFMINBCD
		call	BCDBIN2			;von BCD nach binr umwandeln
		movwf	UHRMINUTE		;und die Minute der mitlaufenden Uhr
						;berschreiben

		clrf	UHRSEKUNDE		;Sekunde lschen

		movf	DCFTAGBCD,w		;DCFTAGBCD
		call	BCDBIN2			;von BCD nach binr umwandeln
		movwf	DATUMTAG		;und den Tag des mitlaufenden Datums
						;berschreiben

		movf	DCFMONBCD,w		;DCFMONBCD
		call	BCDBIN2			;von BCD nach binr umwandeln
		movwf	DATUMMONAT		;und den Monat des mitlaufenden Datums
						;berschreiben

		movf	DCFJAHRBCD,w		;DCFJAHRBCD
		call	BCDBIN2			;von BCD nach binr umwandeln
		movwf	DATUMJAHR		;und das Jahr des mitlaufenden Datums
						;berschreiben

		movf	DCFWOCHENTAG,w		;DCFWOCHENTAG
		movwf	DATUMWTAG

		bsf	DCFSTATUS,DCFSYNC

DCFUPMINUTEENDE bcf	DCFSTATUS,DCFFEHLER
		bcf	DCFSTATUS,DCFNEUEMIN 	;Anforderungsflag lschen

		return


;**********************************************************************************************
;** INNERE UHR										     **
;**											     **
;** Aufgabe:										     **
;** 	fr den Fall dass kein gltiges DCF-Telegramm empfangen werden kann sorgt dieses     **
;**	Unterprogramm dafr, dass die Uhrzeit trotzdem jede Sekunde aktualisiert wird. Ist   **
;**	fr eine lngere Zeit kein korrekter DCF-Empfang mglich, so Wrden die Minuten,     **
;**	Stunden, Tage usw. stehen bleiben, da ja im Unterprogramm DCFUPMINUTE nur gltige    **
;**	DCF-Telegramme bernommen werden. 						     **
;**	Dieses Unterprogramm wird also parallel zu den Unterprogrammen fr die DCF-Dekod-    **
;**	ierung aufgerufen.								     **
;**											     **
;** Vorgehensweise:									     **
;** 	Die Sekunden (Register UHRSEKUNDE) um 1 erhhen. Beinhalte dieses Register nun den   **
;**	Wert 60, so beginnt eine neue Minute. Daher die Sekunden lschen und die Minuten     **
;**	(Register UHRMINUTE) um 1 erhhen. Beinhaltet dieses Register nun den Wert 60, so    **
;**	beginnt eine neue Stunde. Daher die Minuten lschen und die Stunden (Register	     **
;**	UHRSTUNDE) um 1 erhhen. Beinhaltet dieses Register nun den Wert 24, so beginnt ein  **
;**	neuer Tag. Daher die Stunden lschen. Ist das Mitzhlen des Datums notwendig, so     **
;**	muss nun ein Register fr den Tag (z. B. DATUMTAG) um 1 erhht werden und dieses     **
;**	berprft werden, wobei diese Prfung nun nicht mehr so einfach ist, da ja jeder     **
;**	Monat unterschiedlich viele Tage besitzt. Erschwerend kommt auch noch hinzu, dass    **
;**	auch die Schaltjahre miteinbezogen werden mssen!				     **
;** 											     **
;** Achtung: 										     **
;**	Dieses Unterprogramm muss daher jede Sekunde aufgerufen werden			     **
;**********************************************************************************************
INNEREUHR	incf	UHRSEKUNDE,f		;Sekundenzhler um 1 erhhen
		movf	UHRSEKUNDE,w
		sublw	.60
		btfss	STAT,Z			;Prfen, ob Sekunde=60
		goto	INNERERUHRFERTIG
		clrf	UHRSEKUNDE		;Wenn Sekunde=60, Sekunde lschen
		incf	UHRMINUTE,f		;  und Minute um 1 erhhen
		movf	UHRMINUTE,w
		sublw	.60
		btfss	STAT,Z			;Prfen, ob Minute=60
		goto	INNERERUHRFERTIG
		clrf	UHRMINUTE		;Wenn Minute=60, Minute lschen
		incf	UHRSTUNDE,f		;  und Stunde erhhen
		movf	UHRSTUNDE,w
		sublw	.24
		btfsc	STAT,Z			;Prfen, ob Stunde=24
		clrf	UHRSTUNDE		;Wenn Stunde=24, Stunden lschen
INNERERUHRFERTIG
		return


;******************************** Wanduhrspezifisches Unterprogramm ***************************

;**********************************************************************************************
;** ANZEIGE										     **
;**							 				     **
;** Dieses Unterprogramm wird vom Hauptprogramm alle 4ms aufgerufen			     **
;**											     **
;** Aufgabe und Vorgehensweise:								     **
;**	Wenn die mitlaufende Uhr mit der DCF-Uhr synchronisiert ist (Flag DCFSYNC im Register**
;**	DCFSTATUS gesetzt) die Uhrzeit wie folgt ausgeben. Andenfalls dieses Unterprogramm   **
;**	vorzeitig beenden: Stunde am Port A und abwechselnd Minute und Sekunde am Port B     **
;**	ausgeben. Abwechselnd bedeutet hier, dass beim Aufruf dieses Unterprogramms entweder **
;**	die Minuten am Port B ausgegeben werden oder die Sekunden. Diese Methode entspricht  **
;**	einer einfachen Multiplex-Ansteuerung. Die Unterscheidung, ob die Minuten oder die   **
;**	Sekunden ausgegeben werden hngt vom Bit 0 des Registers FLAGS ab. Dieses Bit wird   **
;**	bei jedem Aufruf invertiert (getoggelt). Besitzt es danach den Wert 1, so werden die **
;**	Minuten angezeigt. Beim nchsten Aufruf dieses Unterprogramms (in ca. 4ms) werden    **
;**	dann die Sekunden angezeigt, da dieses Bit ja wieder invertiert wurde und daher den  **
;**	Wert 0 besitzt.									     **
;**	Mit Hilfe der beiden Tabellen TABSTUNDEN und TABMINUTENSEKUNDEN erfolgt eine An-     **
;**	passung an die Hardware (Umwandlung von 24 Stunden auf 12 Stunden, LED-Matrix am     **
;**	Port B).									     **
;**********************************************************************************************
ANZEIGE		btfss	DCFSTATUS,DCFSYNC	;Ist die mitlaufende Uhr mit der DCF-Uhr
						;  synchronisiert?
		goto	ANZEIGEENDE		;nein: Unterprogramm vorzeitig beenden

		movf	UHRSTUNDE,w		;ja: Stunde mit Hilfe der Tabelle TABSTUNDE
		andlw	b'00011111'
		call	TABSTUNDEN		;  von 0-23 auf 0-11 umwandeln und um eine
						;  Stelle verschieben  
		movwf	PORTA			;  und am Port A ausgeben

		btg	FLAGS,0	
		btfss	FLAGS,0			;Abwechselnd
		goto	ANZEIGESEK

ANZEIGEMIN	movf	UHRMINUTE,w		;  Minute
		goto	ANZEIGEW1

ANZEIGESEK	movf	UHRSEKUNDE,w		;  oder Sekunde

ANZEIGEW1	andlw	b'00111111'
		call	TABMINUTENSEKUNDEN	;  so anpassen, dass die richtige
		movwf	PORTB			;  Leuchtdiode am Port B leuchtet

ANZEIGEENDE	return	


;******************************** Weitere, allgemeine Routinen ********************************

;**********************************************************************************************
;** Umwandlung von BCD nach binr:							     **
;** Aufgabe:										     **
;**	+ Die im w-Register stehende 2-stellige-BCD-Zahl nach binr umwandeln		     **
;**											     **
;** Vorgehensweise:									     **
;**	+ Die umzuwandelnde BCD-Zahl im temporren Register TEMP1 zwischenspeichern	     **
;**	+ w-Register (Arbeitsregister) lschen.						     **
;**	+ Die BCD-Zahl bitweise berprfen.						     **
;**	  z .B: BCD-Zahl: 0010 0110  (= 26)						     **
;**			  |||| |||| 							     **
;**			  |||| ||| -   0  ( = 0 x 1)					     **
;**			  |||| || -- + 2  ( = 1 x 2)					     **
;**			  |||| | --- + 4  ( = 1 x 4)					     **
;**			  ||||  ---- + 0  ( = 0 x 8)					     **
;**			  ||| ------ + 0  ( = 0 x 10)					     **
;**			  || ------- +20  ( = 1 x 20)					     **
;**			  | -------- + 0  ( = 0 x 40)					     **
;**			   --------- + 0  ( = 0 x 80)					     **
;**			------------------------------					     **
;**			             =26						     **
;**	+ Ergebnis in w									     **
;**											     **
;** Anmerkung: Das temporre Register TEMP1 wird hier nur zum Zwischenspeichern bentig      **
;**	und kann daher auch wo anders verwendet werden.					     **
;**********************************************************************************************
BCDBIN2		movwf	TEMP1		;umzuwandelnde BCD-Zahl zwischenspeichern
		clrw			;Arbeitsregister (w-Register) lschen
		btfsc	TEMP1,0		;Ist Bit0 der BCD-Zahl gesetzt,
		addlw	.1		;  zum w-Register den Wert 1 addieren
		btfsc	TEMP1,1		;Ist Bit1 der BCD-Zahl gesetzt,
		addlw	.2		;  zum w-Register den Wert 2 addieren
		btfsc	TEMP1,2		;Ist Bit2 der BCD-Zahl gesetzt,
		addlw	.4		;  zum w-Register den Wert 4 addieren
		btfsc	TEMP1,3		;Ist Bit3 der BCD-Zahl gesetzt,
		addlw	.8		;  zum w-Register den Wert 8 addieren
		btfsc	TEMP1,4		;Ist Bit4 der BCD-Zahl gesetzt,
		addlw	.10		;  zum w-Register den Wert 10 addieren
		btfsc	TEMP1,5		;Ist Bit5 der BCD-Zahl gesetzt,
		addlw	.20		;  zum w-Register den Wert 20 addieren
		btfsc	TEMP1,6		;Ist Bit6 der BCD-Zahl gesetzt,
		addlw	.40		;  zum w-Register den Wert 40 addieren
		btfsc	TEMP1,7		;Ist Bit7 der BCD-Zahl gesetzt,
		addlw	.80		;  zum w-Register den Wert 80 addieren, im w-Register
					;  befindet sich nun das Ergebnis
		return


;******************************** Hauptprogramm ***********************************************

;**********************************************************************************************
;** Aufgaben des Hauptprogramms:							     **
;**	+ Aufrufen von INIT (Initialisierungen)						     **
;**	+ Interrupt freigeben								     **
;**	+ Ttigkeiten und/oder Unterprogramme, die alle 4ms durchgefhrt werden mssen	     **
;**		+ Unterprogramm DCFROUTINE aufrufen					     **
;**		+ Unterprogramm ANZEIGE aufrufen					     **
;**	+ Ttigkeiten und/oder Unterprogramme, die jede Sek. durchgefhrt werden mssen	     **
;**		+ Unterprogramm INNEREUHR aufrufen					     **
;**	+ Bei einem Minutenwechsel (Flag DCFNEUEMIN ist gesetzt) das Unterprogramm	     **
;**	  DCFUPMINUTE abarbeiten							     **
;**	+ Bei einem Sekundenwechsel (Flag DCFNEUESEK ist gesetzt) das Unterprogramm	     **
;**	  DCFUPSEKUNDE abarbeiten							     **
;**********************************************************************************************
Beginn		call	INIT			;Initialisierungen
		movlw	b'10100000'		;TMR0 freigeben durch
		movwf	INTCON			;Setzen von GIE und T0IE im Register INTCON

HPSCHLEIFE	btfsc	FLAGSISRHP,FLAG4MSEK
		goto	HPWEITER1

		btfsc	FLAGSISRHP,FLAG1SEK
		goto	HPWEITER2

		btfsc	DCFSTATUS,DCFNEUEMIN 	;Wenn Anforderungsflag DCFNEUEMIN gesetzt
		call	DCFUPMINUTE		;Unterprogramm DCFUPMINUTE ausfhren
		btfsc	DCFSTATUS,DCFNEUESEK 	;Wenn Anforderungsflag DCFNEUESEK gesetzt
		call	DCFUPSEKUNDE		;Unterprogramm DCFUPSEKUNDE ausfhren

		goto	HPSCHLEIFE

HPWEITER1	call	DCFROUTINE		;Ttigkeiten, die alle 4 ms durchgefhrt
						;  werden mssen

		call	ANZEIGE
		bcf	FLAGSISRHP,FLAG4MSEK	;Anforderungsflag (fr 4ms-Aktivitaeten)
						;  zurcksetzen
		goto	HPSCHLEIFE

HPWEITER2	call	INNEREUHR		;Ttigkeiten, die jede Sekunde durchgefhrt
						;  werden mssen

		bcf 	FLAGSISRHP,FLAG1SEK	;Anforderungsflag (fr 1-Sek.-Aktivitaeten)
						;  zurcksetzen
		goto	HPSCHLEIFE

		end
	
